#include <pwd.h>
#include <time.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <syslog.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#define SCHAR_MAX	127
#define SHRT_MAX	32767
#define INT_MAX	2147483647
#define LONG_MAX	2147483647L
#define OUT_DEF  1000000
#define WORDSIZE __WORDSIZE/4

/**************************************************************************
 * INITIALIZATION
 *  - variables must be initialize before used
 *  
 * CAST
 *  - cast from a diffent(less) type (this can cause buffer overflow or underflow) 
 *
 * PARAMETER VALIDATION
 *  - input validation
 *  - format string
 *  - buffer overflow/underflow
 *  - array out of bound
 *  - relational operator in conditional statements
 *  - division by zero
 *  - loops
 * 
 * RETRUN VALUES
 *  - check if is the right value
 *
 * MEMORY MANAGEMENT
 *  - double free
 *  - never release memory
 *  - never allocate memory
 *  - release memory and used
 *  - never release some which never been allocated
 *
 * MORE SAFETY FUNCTIONS WITH VALUE RANGE
 *  - strcpy -> strncpy (..)
 *
 * FILE ACCESS
 *  - check if exist
 *  - check permissions
 *  - check if the file is not a symbolic link
 *  - use the descripter, instead of const name
 *  - never use access() function
 *  - a program must not pass the Path in two or more functions as:
 *    .open, stat, symlink, umount, unlink
 *  - must do chdir() after chroot()
 *  - create temp files in directories that are not share (! \tmp , \var\tmp)
 *  - never use tempnam, tmpfile, tmpnam, mktemp
 *  - before use mkstemp use umask(077)
 *
 * LEAST PRIVILEGES
 *  - system calls
 *    . execl, execv, popen, system
 *  - when create a child (fork), the child will have the same privileges as father
 *
 * CRIPTOGRAFY
 *  - random numbers
 *   . get good seeds
 *   . never used rand and random with a constant seed
 *   . try to use /dev/(u)random
 *
 * LIBRABRIES
 * 
 * TRY CASH EXCEPTIONS
 * 
 *
 **************************************************************************/

/**************************************************************************
 * INITIALIZATION
 * func_1_**()
 **************************************************************************
 *  - variables must be initialize before used
 **************************************************************************/
void func_1_01 (void) {
  int i;
  int j;						// vulnerability
  for (i = 0; i < 10; i++) {
		for ( ; j > 0; j--) {
			printf("func_1_01");
		}
  }
}

void func_1_02 (void) {
  char *str1;					// vulnerability
  char *str2;
  printf("String 1: %s", str1); 
  str2 = (char *) malloc (10);
  str2 = "string 2";
  printf("String 2: %s", str2);
  //free(str2); miss free		// vulnerability
}

void func_1_03 (void) {
  char 		      *str = "WELL DONE";
  int             a, b, c, d, e, f, g, h, i;
  unsigned short j;			// vulnerability
  a = 0; b = 1;
  c = b + 1; d = b + c;
  e = j;						// vulnerability
  printf("First%cword: %c%c%c%c", str[e], str[a], str[b], str[c], str[d]);  
  f = 5; g = 6; h = 7; i = 8;
  printf("Second%cword: %c%c%c%c", str[e], str[f], str[g], str[h], str[i]); 

  while ( j <= 8 )				// vulnerability
    printf("%c", str[j]);
}

void aux_func_1_04(char* str){
	str = malloc(10);
	str = "ollllla";
	printf(str);
}

void func_1_04(void){
  char *str;
  aux_func_1_04(str);
}

void aux_func_1_05(char* str){
	printf(str); 						//vulnerability
}

void func_1_05(void){
  char str[10];						//vulnerability
  aux_func_1_04(str);
}

/**************************************************************************
 * CAST
 * func_2_**()
 **************************************************************************
 *  - cast from a diffent(less) type (this can cause buffer overflow or underflow) 
 **************************************************************************/
void func_2_01 (void) {
  int   si_a = 28;
  int   si_b = 80;
  float sf_f;

  sf_f = si_a / 
         si_b;			// vulnerability - miss cast in variable 'a' and 'b'
}

void func_2_02 (void) {
  int   si_a = 28;
  int   si_b = 80;
  float sf_f;

  sf_f = (float)si_a / 
         (float)si_b;
}

void func_2_03 (void) {
  int             si_a = 28000;
  int             si_b = 80000;
  unsigned short us_s;

  us_s = (unsigned short)si_b / 
         (unsigned short)si_a; 	// vulnerability - overflow variable b
}

void func_2_04 (char *str) {
  char sc_c = (int) str; 			// vulnerability

  printf("%c",sc_c);
}

/**************************************************************************
 * PARAMETER VALIDATION
 * func_3_**()
 **************************************************************************
 *  - input validation (format string, buffer ... )
 *  - format string
 *  - buffer overflow/underflow
 *  - array out of bound
 *  - integer overflow/underflow
 *  - relational operator in conditional statements
 *  - division by zero
 *  - loops
 **************************************************************************/
/**
 * Input validation
 * - Format String
 */
void func_3_01 (void) {
  char str_scanf     [10];
  char str_sprintf   [10];
  char fstr_untrusted[10];
  char fstr___trusted[10] = "%9s";

  // get input
  fgets(fstr_untrusted, sizeof(fstr_untrusted), stdin); //vulnerability

  // use format strings for reading from stdin
  scanf("%s",           str_scanf);	// vulnerability
	scanf("%9s",          str_scanf);
  scanf(fstr___trusted, str_scanf);
  scanf(fstr_untrusted, str_scanf); // vulnerability

  // use format strings for reading from str_scanf
	sprintf(str_sprintf,  "%s",                str_scanf); // vulnerability
  sprintf(str_sprintf,  "9%s",               str_scanf);
  sprintf(str_sprintf,  fstr___trusted,      str_scanf);
  sprintf(str_sprintf,  fstr_untrusted,      str_scanf); // vulnerability
  snprintf(str_sprintf, sizeof(str_sprintf), fstr_untrusted, str_scanf); // vulnerability

  // use format strings for writing to syslog
  syslog(LOG_ERR, "%s\n",         str_scanf); // vulnerability
	syslog(LOG_ERR, "%9s\n",        str_scanf);
  syslog(LOG_ERR, fstr___trusted, str_scanf);
  syslog(LOG_ERR, fstr_untrusted, str_scanf); // vulnerability
}

/**
 * Input validation
 * - Buffer Overflow
 */
void func_3_02 (void) {
  char str__gets_nok[10];
  char str_fgets__ok[10];
  char str_fgets_nok[10];
  char str_scanf__ok[10];
  char str_scanf_nok[10];

  gets (str__gets_nok); 								  									 // vulnerability
	fgets(str_fgets__ok, sizeof(str_fgets__ok) - 1,    stdin);
  fgets(str_fgets__ok, sizeof(str_fgets__ok),        stdin); // vulnerability
  fgets(str_fgets_nok, sizeof(str_fgets__ok) + 10,   stdin); // vulnerability
  
  scanf("%9s",  str_scanf__ok);
  scanf("%10s", str_scanf_nok); // vulnerability - illegal			
  scanf("%s",   str_scanf_nok); // vulnerability - possible overflow  
}

void func_3_03 (void) {
  char printf[10], execv[10];
  fgets (printf, sizeof(printf), stdin);//vulnerability
  strcpy(execv,  printf);
}

/**
 * Input validation
 * - Array out of bound
 */
void func_3_04 (void) {
  char *str[] = {"sub", "exp", "div"};
  char *p_str;							// vulnerability
  int  index;
  scanf("%d", &index); 					// vulnerability - must be between 0 .. 2
  p_str = str[index];					// vulnerability
}

void func_3_05 (void) {
  char *str[] = {"sub", "exp", "div"};
  char *p_str;							// vulnerability
  int  index;
  scanf("%d", &index); 					// vulnerability - must be between 0 .. 2
  if(index >= 0 && index <= 2)
  	p_str = str[index];
}

/**
 * Input validation
 * - Integer overflow/underflow
 */
void func_3_06 (void) {
  unsigned short number_1;
  unsigned short number_2;
  unsigned short result_1;
  unsigned short result_2;
  scanf("%hi",   &number_1);      // vulnerability - possible overflow or underflow
  scanf("%hi",   &number_2);	  	// vulnerability
  result_1 = number_1 * number_2; // vulnerability - integer overflow
  result_2 = number_1 - number_2; // vulnerability - integer underlfow
}

void func_3_07 (void) {
  unsigned short const_num = 65535;
  unsigned short result_1  = 0;
  unsigned short result_2  = 0;
  unsigned short number_1;
  unsigned short number_2;
  char           number[6];
  fgets(number, 6, stdin);		  // vulnerability
  if ( number[0] != '-' ) {
    number_1 = (unsigned short)atoi(number);

    bzero(number, 6);
    if( fgets(number, 6, stdin) != NULL ) { 

      if ( number[0] != '-' ) {
      	number_2 = (unsigned short)atoi(number);

      	if ( (const_num - number_2) > number_1 ){
        	result_1 = number_1 + number_2; //never integer overflow

        	if ( number_1 >= number_2 )
          	result_2 = number_1 - number_2; //never integer underlfow
				}      
			}
	  }
  }
}

/**
 * Input validation
 * - Division by zero
 */
void func_3_08 (void) {
  int number, result, diff_zero;
  scanf("%d", &number);				// vulnerability
  scanf("%d", &diff_zero);			// vulnerability
  result = number / diff_zero;	// vulnerability
}

void func_3_09 (void) {
  int number, result, diff_zero;
  scanf("%d", &number);			// vulnerability
  scanf("%d", &diff_zero);		// vulnerability
  if ( diff_zero > 0 )
    result = number / diff_zero;
}

/**
 * Format strings
 */
void func_3_10 (void) {
  char str1[] = "First format string";
  char str2[] = "Second format string";
  char *p     = str1;
  printf(p);
  printf(str2);
}

void func_3_11 (void) {
  char buf[10];
  if ( fgets (buf, sizeof(buf), stdin)!= NULL ){
    printf(buf);		// vulnerability
    printf("%s", buf);
  } else {
    printf("ERROR:: on stdin!");
  }
}

void func_3_12 (void) {
  char *msg = "test";
  fprintf(stdout ,"fingerd: %s: \r\n", msg);
  exit(1);
}

void func_3_13 (void) {
  char  *p, buf[10];
  gets  (p);				// vulnerability
  system(p);				// vulnerability
  strcpy(buf,"date");
  system(buf);
  printf(buf + 5);		
  printf("\n");
  printf("%s", buf + 8);	// vulnerability
  printf("%s", buf + 11);	// vulnerability 
  bzero (buf,  10);
}

void func_3_14 (void) {
  const char *buf = "null terminate";
  char       aux_str[10];
  char       str    [10];
  int        i;
  for(i = 0; i < 4; i++) {  
    str[i] = buf[i];
    aux_str[i] = buf[i];
  }
  
  printf("%s", str);		// vulnerability
  i = 10;
  aux_str[i] = '\0';		// vulnerability
  printf("%s", aux_str);	
}

/**
 * Buffer overflow/underflow
 *
 */
void func_3_15 (void) {
  char dst__ok[20];
  char dst_nok[5];
  char src[] = "1234567890";
    
  // dangerous functions
  strcpy(dst__ok, src);                    
  strcpy(dst_nok, src);                    // vulnerability - dst_nok is too small
  if (sizeof(dst_nok) >= sizeof(src))
    strcpy(dst_nok, src);                  
  strncpy(dst__ok, src, sizeof(dst__ok));
  strncpy(dst_nok, src, sizeof(dst_nok));
  strncpy(dst_nok, src, sizeof(src));     // vulnerability - dst_nok is too small
  memcpy (dst__ok, src, sizeof(dst__ok));
  memcpy (dst_nok, src, sizeof(dst_nok));
  memcpy (dst_nok, src, sizeof(src));     // vulnerability - dst_nok is too small

  // reset
  strcpy(dst__ok, "1");
  strcpy(dst_nok, "1");

  strcat(dst__ok, src);
  strcat(dst_nok, src);					  				// vulnerability
  if (sizeof(dst_nok)-strlen(dst_nok)+1 >= sizeof(src))
    strcat(dst_nok, src);

  // reset
  strcpy(dst__ok, "1");
  strcpy(dst_nok, "1");

  strncat(dst__ok, src, sizeof(dst__ok)-strlen(src)-1);
  strncat(dst__ok, src, sizeof(dst__ok)-strlen(src));	// vulnerability
  strncat(dst_nok, src, strlen(src)); 								// vulnerability
  if (sizeof(dst_nok)-strlen(dst_nok)+1 >= sizeof(src))
    strncat(dst_nok, src, strlen(src));
}

void func_3_16 (void) { //underflow
  char str[10] = "OLA";
  int  i       = 0;
  while ( i <= 0 ) {
    str[i] = 'a'; 		// vulnerability
    i--;							// vulnerability
  }
}

void func_3_17 (void) {
  char buf[4];
  char *p;
  p  = buf;
  p += 50; 						// vulnerability
  *p = 'X';						// vulnerability
}

void func_3_18 (void) {
  char buf[4] = "BLA";
  buf[7]      = 'X';	// vulnerability
}

void func_3_19 (void) {
  char buf[1024];
  int  i = -1;
  char c;
  while ( (c = getchar()) != '\n') {
    if ( c == -1 )
      break;
    buf[++i] = c;			// vulnerability
  }
}

/**
 *  Array out of bound
 *  - read/write out of bounds in stack (access through array/pointer)
 */
void func_3_20 (void) {
  int array[8] = { 0, 1, 2, 3, 4, 5, 6, 7}; // A global array
  int n        = -1;
    
  if(array[n] != 0)  	// vulnerability
    printf("%d", n);
  if(array[-5] != 0) 	// vulnerability
    printf("-5%d", n);
  if(array[2] != 7) 
    printf("2");
}

void func_3_21 (void) {
  char *str = "booomm";
  int  i    = 0;
  while(str[i] != '*') {	// vulnerability
    i++;
    str[i] = 'x';			// vulnerability
  }
}

void func_3_22 (void) {
  int value;
  int array[10];
  int OUT_VAR = OUT_DEF;

  // array-form: read
  value = array[0];
  value = array[10];      // vulnerability - out of bounds
  value = array[OUT_DEF]; // vulnerability - way out of bounds!
  value = array[OUT_VAR]; // vulnerability - way out of bounds!
  value = array[rand()];  // vulnerability - maybe out of bounds!
  
  // array-form: write
  value          = 10;
  array[0]       = value;
  array[10]      = value; // vulnerability - out of bounds
  array[OUT_DEF] = value; // vulnerability - way out of bounds!
  array[OUT_VAR] = value; // vulnerability - way out of bounds!
  array[rand()]  = value; // vulnerability - maybe out of bounds!

  // pointer-form: read
  value = (int) array + (0*sizeof(int));
  value = (int) array + (10*sizeof(int));     // vulnerability - out of bounds
  value = (int) array + (OUT_DEF*sizeof(int));// vulnerability - way out of bounds!
  value = (int) array + (OUT_VAR*sizeof(int));// vulnerability - way out of bounds!
  value = (int) array + (rand()*sizeof(int)); // vulnerability - maybe out of bounds!

  // pointer-form: write
  value            = 10;
  *(array+0)       = value;
  *(array+10)      = value; // vulnerability - out of bounds
  *(array+OUT_DEF) = value; // vulnerability - way out of bounds!
  *(array+OUT_VAR) = value; // vulnerability - way out of bounds!
  *(array+rand())  = value; // vulnerability - maybe out of bounds!
}

/**
 *  Array out of bound
 *  - read/write out of bounds in heap (access through array/pointer)
 */
void func_3_23 (void) {
  int value;
  int *array  = (int *) malloc(10 * sizeof(int));
  int OUT_VAR = OUT_DEF;

  // array-form: read
  value = array[0];
  value = array[10];      // vulnerability - out of bounds
  value = array[OUT_DEF]; // vulnerability - way out of bounds!
  value = array[OUT_VAR]; // vulnerability - way out of bounds!
  value = array[rand()];  // vulnerability - maybe out of bounds!
  
  // array-form: write
  value          = 10;
  array[0]       = value;
  array[10]      = value; // vulnerability - out of bounds
  array[OUT_DEF] = value; // vulnerability - way out of bounds!
  array[OUT_VAR] = value; // vulnerability - way out of bounds!
  array[rand()]  = value; // vulnerability - maybe out of bounds!

  // pointer-form: read
  value = (int) array + (0*sizeof(int));
  value = (int) array + (10*sizeof(int));      // vulnerability - out of bounds
  value = (int) array + (OUT_DEF*sizeof(int)); // vulnerability - way out of bounds!
  value = (int) array + (OUT_VAR*sizeof(int)); // vulnerability - way out of bounds!
  value = (int) array + (rand()*sizeof(int));  // vulnerability - maybe out of bounds!

  // pointer-form: write
  value            = 10;
  *(array+0)       = value;
  *(array+10)      = value; // vulnerability - out of bounds
  *(array+OUT_DEF) = value; // vulnerability - way out of bounds!
  *(array+OUT_VAR) = value; // vulnerability - way out of bounds!
  *(array+rand())  = value; // vulnerability - maybe out of bounds!

  free(array);
}


/**
 *  Integer overflow/underflow
 *  - underflow/overflow of different datatypes (access through variable/pointer)	
 */
void func_3_24 (void) {
  // variables
  signed char    _char;
  signed short   _short;
  signed int     _int;
  signed long    _long;

  unsigned char  _uchar;
  unsigned short _ushort;
  unsigned int   _uint;
  unsigned long  _ulong;

  // pointers
  signed char    *p_char   = &_char;
  signed short   *p_short  = &_short;
  signed int     *p_int    = &_int;
  signed long    *p_long   = &_long;

  unsigned char  *p_uchar  = &_uchar;
  unsigned short *p_ushort = &_ushort;
  unsigned int   *p_uint   = &_uint;
  unsigned long  *p_ulong  = &_ulong;

  /**
   * Underflow
   */
  // variable reset
  _char     = CHAR_MIN;
  _short    = SHRT_MIN;
  _int      = INT_MIN;
  _long     = LONG_MIN;
  _uchar    = 0;
  _ushort   = 0;
  _uint     = 0;
  _ulong    = 0;

  // variable decrement
  _char   -= 10;	// vulnerability
  _short  -= 10;	// vulnerability
  _int    -= 10;	// vulnerability
  _long   -= 10;	// vulnerability
  
  _uchar  -= 10;	// vulnerability
  _ushort -= 10;	// vulnerability
  _uint   -= 10;	// vulnerability
  _ulong  -= 10;	// vulnerability

  // variable reset
  _char     = CHAR_MIN;
  _short    = SHRT_MIN;
  _int      = INT_MIN;
  _long     = LONG_MIN;
  _uchar    = 0;
  _ushort   = 0;
  _uint     = 0;
  _ulong    = 0;

  // pointer decrement
  *p_char   -= 10;	// vulnerability
  *p_short  -= 10;	// vulnerability
  *p_int    -= 10;	// vulnerability
  *p_long   -= 10;	// vulnerability

  *p_uchar  -= 10;	// vulnerability
  *p_ushort -= 10;	// vulnerability
  *p_uint   -= 10;	// vulnerability
  *p_ulong  -= 10;	// vulnerability

  /**
   * Overflow
   */
  // variable reset
  _char     = CHAR_MAX;
  _short    = SHRT_MAX;
  _int      = INT_MAX;
  _long     = LONG_MAX;
  _uchar    = UCHAR_MAX;
  _ushort   = USHRT_MAX;
  _uint     = UINT_MAX;
  _ulong    = ULONG_MAX;

  // variable increment
  _char   += 10;	// vulnerability
  _short  += 10;	// vulnerability
  _int    += 10;	// vulnerability
  _long   += 10;	// vulnerability

  _uchar  += 10;	// vulnerability
  _ushort += 10;	// vulnerability
  _uint   += 10;	// vulnerability
  _ulong  += 10;	// vulnerability

  // variable reset
  _char     = CHAR_MAX;
  _short    = SHRT_MAX;
  _int      = INT_MAX;
  _long     = LONG_MAX;
  _uchar    = UCHAR_MAX;
  _ushort   = USHRT_MAX;
  _uint     = UINT_MAX;
  _ulong    = ULONG_MAX;

  // pointer increment
  *p_char   += 10;	// vulnerability
  *p_short  += 10;	// vulnerability
  *p_int    += 10;	// vulnerability
  *p_long   += 10;	// vulnerability

  *p_uchar  += 10;	// vulnerability
  *p_ushort += 10;	// vulnerability
  *p_uint   += 10;	// vulnerability
  *p_ulong  += 10;	// vulnerability
}

/**
 *  Integer overflow/underflow
 *  - underflow/overflow of different datatypes (access through casted pointer)
 */
void func_3_25 (void) {
  // variables
  unsigned char  _uchar    = 0;
  unsigned short _ushort   = 0;
  unsigned int   _uint     = 0;
  unsigned long  _ulong    = 0;

  // pointers
  signed char    *p_char   = (signed char *)  &_uchar;
  signed short   *p_short  = (signed short *) &_ushort;
  signed int     *p_int    = (signed int *)   &_uint;
  signed long    *p_long   = (signed long *)  &_ulong;

  // casted pointer decrement
  *p_char   -= 10;	// vulnerability
  *p_short  -= 10;	// vulnerability
  *p_int    -= 10;	// vulnerability
  *p_long   -= 10;	// vulnerability

  *p_char    = 0;
  *p_short   = 0;
  *p_int     = 0;
  *p_long    = 0;

  *p_char   += 10;	// vulnerability
  *p_short  += 10;	// vulnerability
  *p_int    += 10;	// vulnerability
  *p_long   += 10;	// vulnerability
}

/**
 * Relational operator in conditional statements
 */
void func_3_26 (void) { //overflow
  char str[10] = "Hello";
  int  i       =  0;

  while ( i <= 10 ) { 							// vulnerability - the operator equal 
		if ( str[i] != 'e' && str[i] != 'o' )	// vulnerability
			str[i] = 'a';						// vulnerability
		i++;
  }
}

void func_3_27 (void) { //overflow
  char str[10] = "Hello";
  int  i       = 0;

  while ( i < 15 ) { 							// vulnerability
		if ( str[i] != 'e' && str[i] != 'o' )	// vulnerability
			str[i] = 'a';						// vulnerability
		i++;
  }
}

void func_3_28 (void) {
  int i;
  int *p1 = malloc (10 * sizeof(int));
  int *p2 = malloc (10 * sizeof(int));
  for ( i = 0; i < 10; i++ )
    p2[i] = 2;
  for ( i = 0; i < 15; i++ )	// vulnerability
    p1[i] = 3;					// vulnerability

  free(p1);
  free(&p2);					// vulnerability
}

void func_3_29 (void) {
  int F_SYS = 0;
  int root  = 0;
  //Compiler detect this problem
  //if F_SYS or root use the operater '='
  //but with -wall detect
  if ( root == 0 && F_SYS == 1 )
    root = 1;
}

void func_3_30 (void) {
  int  i;
  char *name = "Code Security";

  //The operater '&&' should be '||'
  for(i = 0; *name == 'c' && i != (strlen(name) - 1); i++)	// vulnerability
    printf("No 'c' found yet!");
}

/**
 * Division by zero
 */
//detect by compiler -Wall
/*int func_3_31 (int num) {
  return num / 0;
}*/

int func_3_32 (int num) {
  int zero = 0;
  return num / zero;						// vulnerability
}

int func_3_33 (int number, int d) {
  if ( d == 0 )
	return -1;
  return (number * number) / (number * d);
}

int func_3_34 (int number, int d) {
  return (number * number) / (number * d);	// vulnerability
}

//compiler detect -Wall
/*int func_3_35 (int number) {
	return (number * number) / (number * 0);
}*/

/**
 * Loops
 */
void func_3_36 (void) {
  int i;					// vulnerability
  for ( ; i <= 10; i++ )	// vulnerability
  printf("HELLO");
}

void func_3_37 (void) {
  int i = 6;
  int j = 5;
  i += j;
  while ( i != 10 ) {	// vulnerability
    j *= i;				// vulnerability
    i++;				// vulnerability
  }
}

int aux_func_3_38 (int i) {
  if ( i == 0 )
    return 1;
  else
    aux_func_3_38(--i); // vulnerability - must be i--
  return 0;
}

void func_3_38 (void) {
  aux_func_3_38(10);
}

/**************************************************************************
 * RETURN VALUES
 * func_4_**()
 **************************************************************************
 *  - check if is the right value
 **************************************************************************/
int *aux_func_4_01 (int i) {
  int p[10];
  p[i] = 1;						// vulnerability
  return (int *)p[i];			// vulnerability
}
 
void func_4_01 (void) {
  int *pi = aux_func_4_01(4);
  int i   = *pi;
  printf("the number at 4 is %d", i);
}

char aux_func_4_02 (int i) {
  char   *array = "this is a array";
  return array[i];
}

void func_4_02 (void){
  int i;
  for( i = 0; i < 10; i++ )
    aux_func_4_02(i);
}

int aux_func_4_03 (int m, int n) {
  int calc = (m * n) / n;
  if ( calc >= 5 )
    return calc;
  return -1;
}

void func_4_03 (void) {
  int  index;
  char *str = "some phrase to ...";
  index = aux_func_4_03(5, 4);
  printf("%c", str[index]);
  index = aux_func_4_03(4, 4);
  printf("%c", str[index]);		// vulnerability
}

/**************************************************************************
 * MEMORY MANAGEMENT
 * func_5_**()
 **************************************************************************
 *  - double free
 *  - never release memory
 *  - never allocate memory
 *  - release memory and used
 *  - never release some which never been allocated
 **************************************************************************/

/**
 * - double free
 */
void aux_func_5_01 (char *buf) {
  char *abc = "------- testing -------";
  int  j = 0;
  while (abc++ != '\0') // vulnerability - loop :: *abc++
    j++;				 // vulnerability
  printf("%d", j);
  free(buf);
}

void func_5_01 (void) {
  char *buf;
  int  i;
  buf = (char *) malloc(256);
  //(...)

  for (i = 0; i<= 10; i++)
    printf("%c", buf[i]);

  aux_func_5_01(buf);

  free(buf);			// vulnerability - double free
}

/**
 * - never release memory
 */
void func_5_02(void) {
  int  a    = 1;
  int  d, b;
  char *p   = (char*) malloc (10);
  char *ptr = NULL;
  char str[50];

  d = b * 2;					// vulnerability - initialization

  for(a = 0; a < 100; a++)
    ptr = (char*) malloc(200);	// vulnerability - create and no free

  printf("%c", str[51]);		// vulnerability
  printf("%s", p);				// vulnerability - no free (p)
}

/**
 * - never allocate memory
 */
void func_5_03 (void) {
  int *p1 = NULL, *p2;
  p1      = (int *) malloc (sizeof(int));
  *p2     = 65;					// vulnerability
  free(p1);
}

void func_5_04 (void) {
  int  i;
  char *s;						// vulnerability
  for ( i = 0; i <= 10; i++ )
    printf("%c", s[i]);
}

/**
 * - release memory and used
 */
void func_5_05 (void) {
  char *str = (char*) malloc (256);
  fgets (str,  256, stdin);
  printf("%s", str);
  free  (str);
  printf("One more time: %s", str);	// vulnerability
}

/**
 * - never release some, which never been allocated
 */
void func_5_06 (void) {
  char b = 'b';
  int  a = 33;
  printf("%d %c", a, b);
  free  (&b);						// vulnerability
  free  (&a);						// vulnerability
}

void func_5_07 (void){
  char *s = "Double Free";
  char *m;
  char *t;							// vulnerability
  int  count;
  m = malloc(1024);
  fgets (m, 100, stdin);
  // if m assigmnent to something then in the 2nd printf, we'll have illegal access
  printf("This is possible! %s", s);
  free  (m);
  fgets (t, sizeof(t), stdin);		// vulnerability - miss malloc for 't'
  while ( count != 10 ) {			// vulnerability - loop
    //is possible access a region out of m
    printf("%c",       *(m+count));	// vulnerability 
    printf("%.10d \n", count--);
  }
  free(m);							// vulnerability
}

void func_5_free (int *pointer_fnc_freed) {
  free(pointer_fnc_freed);
}

void *func_5_malloc (size_t size) {
  return malloc(size);
}

void func_5_08 (void) {
  unsigned int i = 0;
  int           array[10];
  /**
   * Mallocs
   */
  int *pointer__malloc__free = (int *) malloc(10 * sizeof(int));
  int *pointer__malloc______ = (int *) malloc(10 * sizeof(int));       // vulnerability (1)
  int *pointer__________free;
  int *pointer_fmalloc__free = (int *) func_5_malloc(10 * sizeof(int));
  int *pointer_fmalloc______ = (int *) func_5_malloc(10 * sizeof(int));// vulnerability (2)
  int *pointer__malloc_ffree = (int *) malloc(10 * sizeof(int));       // vulnerability (3)
  int *pointer_________ffree;
  int *pointer_fmalloc_ffree = (int *) func_5_malloc(10 * sizeof(int));

  // simple write
  for (i = 0; i<10; i++) {
    array[i]                 = 10;
    pointer__malloc__free[i] = 10;
    pointer__malloc______[i] = 10;
    pointer__________free[i] = 10; // vulnerability
    pointer_fmalloc__free[i] = 10;
    pointer_fmalloc______[i] = 10;
    pointer__malloc_ffree[i] = 10;
    pointer_________ffree[i] = 10; // vulnerability
    pointer_fmalloc_ffree[i] = 10;
  }
  /**
   * Frees
   */
  free(array);                       // vulnerability
  free(pointer__malloc__free);
  //free(pointer__malloc______);
  free(pointer__________free);       // vulnerability
  free(pointer_fmalloc__free);
  //free(pointer_fmalloc______);
  func_5_free(pointer__malloc_ffree);
  func_5_free(pointer_________ffree);// vulnerability
  func_5_free(pointer_fmalloc_ffree);

  // simple write
  for (i = 0; i<10; i++) {
    array[i]                 = 10;
    pointer__malloc__free[i] = 10; // vulnerability
    pointer__malloc______[i] = 10;
    pointer__________free[i] = 10; // vulnerability
    pointer_fmalloc__free[i] = 10; // vulnerability
    pointer_fmalloc______[i] = 10;
    pointer__malloc_ffree[i] = 10; // vulnerability
    pointer_________ffree[i] = 10; // vulnerability
    pointer_fmalloc_ffree[i] = 10; // vulnerability
  }// (1)(2) vulnerability - MISS FREE
}

void func_5_09 (void) {
  int *temp;
  /**
   * Mallocs
   */
  int *pointer__malloc__free__free  = (int *) malloc(10 * sizeof(int));
  int *pointer_fmalloc__free__free  = (int *) func_5_malloc(10 * sizeof(int));
  int *pointer__malloc_ffree__free  = (int *) malloc(10 * sizeof(int));
  /**
   * Frees (1)
   */
  temp = pointer__malloc__free__free;
  free(pointer__malloc__free__free);
  pointer__malloc__free__free = temp;
  
  temp = pointer_fmalloc__free__free;
  free(pointer_fmalloc__free__free);
  pointer_fmalloc__free__free = temp;

  temp = pointer__malloc_ffree__free;
  func_5_free(pointer__malloc_ffree__free);
  pointer__malloc_ffree__free = temp;
  /**
   * Frees (2)
   */
  free(pointer__malloc__free__free);   // vulnerability
  free(pointer_fmalloc__free__free);   // vulnerability
  free(pointer__malloc_ffree__free);   // vulnerability
}

void func_5_10 (void) {
  /**
   * Mallocs (1)
   */
  int *pointer__malloc_ffree__malloc  = (int *)        malloc(10 * sizeof(int));
  int *pointer_fmalloc__free__malloc  = (int *) func_5_malloc(10 * sizeof(int));
  int *pointer_fmalloc_ffree_fmalloc  = (int *) func_5_malloc(10 * sizeof(int));
  /**
   * Frees
   */
  func_5_free(pointer__malloc_ffree__malloc);
  free(       pointer_fmalloc__free__malloc);
  func_5_free(pointer_fmalloc_ffree_fmalloc);
  /**
   * Mallocs (2)
   */
  pointer__malloc_ffree__malloc  = (int *)        malloc(10 * sizeof(int));// vulnerability
  pointer_fmalloc__free__malloc  = (int *)        malloc(10 * sizeof(int));// vulnerability
  pointer_fmalloc_ffree_fmalloc  = (int *) func_5_malloc(10 * sizeof(int));// vulnerability
}

void func_5_11 (void) {
  /**
   * Mallocs (1)
   */
  int *pointer__malloc__malloc__free  = (int *)        malloc(10 * sizeof(int));
  int *pointer_fmalloc__malloc__free  = (int *) func_5_malloc(10 * sizeof(int));
  /**
   * Mallocs (2)
   */
  pointer__malloc__malloc__free  = (int *)        malloc(10 * sizeof(int));// vulnerability
  pointer_fmalloc__malloc__free  = (int *)        malloc(10 * sizeof(int));// vulnerability
  /**
   * Frees
   */
  free(pointer__malloc__malloc__free);
  free(pointer_fmalloc__malloc__free);
}

/**************************************************************************
 * MORE SAFETY FUNCTIONS WITH VALUE RANGE
 * func_6_**()
 **************************************************************************
 *  - strcpy -> strncpy
 *  - strcat -> strncat
 *  - printf -> snprintf
 *  - scanf  -> fgets
 **************************************************************************/
void func_6_01 (void) {
  char dest[25];
  char src [75];
  fgets (src,   74, stdin);
  strcpy(dest, src);		// vulnerability
}

void func_6_02 (void) {
  char dest[25];
  char src [75];
  fgets  (src,   74, stdin);
  strncpy(dest, src,    25);// vulnerability
}	

void func_6_03 (void) {
  char str[10];
  scanf("%s", str);			// vulnerability
}

void func_6_04 (void) {
  char str[10];
  fgets(str, 9, stdin);
}

/**************************************************************************
 * FILE ACCESS
 * func_7_**()
 **************************************************************************
 *  - check if exist
 *  - check permissions
 *  - check if the file is not a symbolic link
 *  - use the descripter, instead of const name
 *  - never use access() function
 *  - a program must not pass the Path in two or more functions as:
 *    .open, stat, symlink, umount, unlink
 *  - must do chdir() after chroot()
 *  - create temp files in directories that are not share (! \tmp , \var\tmp)
 *  - never use tempnam, tmpfile, tmpnam, mktemp
 *  - before use mkstemp use umask(077)
 **************************************************************************/
/**
 * - check if exist
 */
//read a file
void func_7_01 (void) {
  FILE *ds;
  char *filename = "users.txt";
  char c;
  ds = fopen(filename, "r");
  while ( (c = fgetc(ds)) != EOF )	// vulnerability - file could not exist
    putchar(c);
  
  fclose(ds);
}

//check if exist and then read a file
void func_7_02 (void) {
  FILE *ds;
  char *filename = "users.txt";
  char c;
  ds = fopen(filename, "r");
  if ( ds != NULL ) {
  	while ( (c = fgetc(ds)) != EOF )
			putchar(c);
  	fclose(ds);	
  }
}

void func_7_03 (char *first, char *second) {
  int a, b;
  a = open(first, O_RDONLY);
  b = open(second, O_RDONLY);
  if ( a == -1 || b == -1 ) {
    perror("open()");
    exit(1);
  }
  close(a);
  b = open(second, O_RDONLY);		// vulnerability
  //close(b);						// vulnerability
}

/**
 *  - check permissions
 */
//TOCTOU
void func_7_04 (void) {
  int b;
  struct stat sbefore, safter;
  lstat("/usr/local/log.txt", &sbefore);
  if ( !access("/usr/local/log.txt", O_RDWR) ) {  // Vulnerability
    b = open("/usr/local/log.txt", O_RDWR);		  // vulnerability
    lstat("/usr/local/log.txt",    &safter);
    if ( safter.st_ino == sbefore.st_ino ) {
      //do some
    }
  }
}

//safe way
void func_7_05 (void) {
  FILE *fs       = NULL;
  char *filename = "/usr/local/log.txt";
  char c;
  int  ds;

  ds = open (filename,  600);
  fchmod(ds, 600);
  fs = fdopen(ds, "rw");
  if ( fs != NULL ) {
  	while ( (c = fgetc(fs)) != EOF )
      putchar(c);
  	fclose(fs);
  }
  fchmod(ds, 666);
}

/**
 *  - check if the file is not a symbolic link
 */
//safe way
void func_7_06 (void) {
  FILE *fs;
  char *filename = "/usr/local/log.txt";
  char buf[10];
  char c;
  int  ds, link;
  ds = open(filename, O_RDWR);

  link = readlink(filename, buf, 9);		// vulnerability
  if ( link == -1 )
    return; 
  fs = fdopen(ds, "rw");
  if ( fs != NULL ) {
    while ( (c = fgetc(fs)) != EOF )
      putchar(c);
    fclose(fs);
  }
}

/**
 *  - use the descripter, instead of const name
 *  &
 *  - a program must not pass the Path in two or more functions as:
 *    .open, stat, symlink, umount, unlink
 */
//bad
void func_7_07 (void) {
  FILE *fs;
  char *filename = "/usr/local/log.txt";
  char c;
  int  ds;

  ds = open (filename, O_RDWR);
  fs = fopen(filename, "rw");		// vulnerability
  if(fs != NULL) {
    while( (c = fgetc(fs)) != EOF )
      putchar(c);
    fclose(fs);
  }
}

//good
void func_7_08 (void) {
  FILE *fs;
  char *filename = "/usr/local/log.txt";
  char c;
  int  ds;
  ds = open (filename, O_RDWR);
  fs = fdopen(ds, "rw");
  if ( fs != NULL ) {
    while ( (c = fgetc(fs)) != EOF )
      putchar(c);
   fclose(fs);
  }
}

/**
 *  - never use access() function
 */
int func_7_09 (void) {
  FILE *fd;
  char *arq = "/bin/pwd.txt"; 
  if ( access(arq, W_OK) == 0 ) {			 // vulnerability
    if ( (fd = fopen(arq, "w")) == NULL ) {  // vulnerability
      perror(arq);
      return 0;
    }
  }
  // Save date
  return 1;
}

int func_7_10 (void) {
  FILE *fd;
  char *arq  = "/bin/pwd.txt";
  char *arq2 = arq;
  if ( access(arq, W_OK) == 0 ) {			// vulnerability
    if ( (fd = fopen(arq2, "w+")) == NULL ) {
      perror(arq);
      return 0;
    }
  }
  //Grava dados em arquivo
  return 1;
}

/**
 *  - must do chdir() after chroot()
 */
//bad
void func_7_11 (void) {
  FILE *f;
  char *file_name;				// vulnerability
  chroot("/home/web");			// vulnerability
  // Miss chdir("/")
  // Miss memory allocation
  scanf("%s", file_name);		// vulnerability - and if the file_name = ../passwd
  f = fopen(file_name, "w");	// vulnerability - miss chdir
}

//good
void func_7_12 (void) { 
  FILE *f;
  char *file_name;				// vulnerability
  chroot("/home/web");//safe
  chdir ("/");
  // We got chdir("/")
  // Miss memory allocation
  scanf("%s", file_name);		// vulnerability - and if the file_name = ../passwd
  f = fopen(file_name, "w");
}

/**
 *  - create temp files in directories that are not share (! \tmp , \var\tmp)
 */
void func_7_13 (void) {
  char   *tempfile = "/tmp/passtmp.txt";
  char   *tempfile2 = "/var/tmp/passtmp.txt";
  mktemp (tempfile);						// vulnerability
  mkstemp(tempfile2);						// vulnerability
  //(...)
}

/**
 *  - never use tempnam, tmpfile, tmpnam, mktemp
 */
void func_7_14 (char *user, char *password) {
  FILE *fd;
  fd = tmpfile();						// vulnerability
  fprintf(fd, "%s %s", user, password);
  // (...)
}

/**
 *  - before use mkstemp use umask(077)
 */
void func_7_15 (void) {
  char *file = "temp.file";
  int  err   = 0;
  err = mkstemp(file);					// vulnerability
  if ( err != -1 ) {
		//(...)
  }
}

void func_7_16 (void) {
  char *file = "temp.file";
  int  err   = 0;
  umask(077); //mask_file & 077 - group and others will be 0
  err = mkstemp(file);
  if ( err != -1 ) {
		// (...)
  }
}

/**************************************************************************
 * LEAST PRIVILEGES
 * func_8_**()
 **************************************************************************
 *  - system calls
 *    . execl, execv, popen, system
 *  - when create a child (fork), the child will have the same privileges as father
 **************************************************************************/
/**
 * -system calls
 */
void aux_func_8_01 (void) { 
  struct passwd *pwd;
  if ( (pwd = getpwuid(getuid())) == NULL )
    return;								// vulnerability - miss drop privilege

  fprintf (stdout, "drop privilege fo %s", pwd->pw_name);
  seteuid (getuid());
}

void func_8_01 (void) {
  aux_func_8_01();
  execl("/bin/sh", "/bin/sh", NULL);		// vulnerability
}

void aux_func_8_02 (void) {
  struct passwd *passwd;
  
  if ( (passwd = getpwuid(getuid())) != NULL ) {
    fprintf(stdout, "drop priv for %s", passwd->pw_name);
    setuid (getuid()); //drop privilege
  }
  execl("/bin/sh", "/bin/sh", NULL);	// vulnerability
}

void func_8_02 (int argc, char *path, char **argv) {
  aux_func_8_02();
  printf("About to exec\n");
  execv (path, argv);					// vulnerability
}

void func_8_03 (void) {
  char *syscall = "/bin/sh";
  //need to drop privileges
  system(syscall);						// vulnerability
}

void func_8_04 (void) {
  char *syscall = "/bin/sh";
  setuid(0); //turn on as superuser
  system(syscall);						// vulnerability
}

void func_8_05 (char **argv) {
  int oldeuid = geteuid();
  // run possible setuid program
  system(argv[0]);						// vulnerability

  // run setuid program
  setuid(0);
  system(argv[0]);						// vulnerability

  // drop priviledges before run possible setuid program
  if ( seteuid(getuid()) )
    perror("ERROR");

  system(argv[0]);

  if ( seteuid(oldeuid) )
    perror("ERROR");

}

/**
 * - when create a child (fork), the child will have the same privileges as father
 */
void func_8_06 (char **argv) {
  // run possible setuid program
  switch ( fork() ) {
    case 0:  // child
      execvp(argv[0], argv); 			// vulnerability
      break;
   	default: sleep(5); break;
  }
}

void func_8_07 (char **argv) {
  // run setuid program
  setuid(0);
  switch ( fork() ) {
    case 0:  // child
      execvp(argv[0], argv); 			// vulnerability
      break;
   	default: sleep(5); break;
  }
}

void func_8_08 (char **argv) {
  int oldeuid = geteuid();
  if ( seteuid(getuid()) )
    perror("ERROR");
  
  switch ( fork() ) {
    case 0:  // child
      execvp(argv[0], argv);
      break;
   	default: sleep(5); break;
  }
  if( seteuid(oldeuid) )
    perror("ERROR");
}

/*************************************************************************
 * CRIPTOGRAFY
 * func_9_**()
 *************************************************************************
 *  - random numbers
 *    . get good seeds
 *    . never used rand and random with a constant seed
 *    . try to use /dev/(u)random
 *************************************************************************/
/**
 * - Random numbers
 */
void func_9_01 (void) {
  int x = rand();				// vulnerability
  printf("%d\n", x);
  
  srand(time(0));
  x = rand();
  printf("%d\n", x);

  srandom(time(0));
  x = random();
  printf("%d\n", x);

  srandom(100);
  x = random();					// vulnerability
  printf("%d\n", x);
}

/**
 * MAIN - Call all functions
 */
int main(int argc, char **argv) {

  // -- INITIALIZATION --
  func_1_01(); printf("func_1_01\n");
  func_1_02(); printf("func_1_02\n");
  func_1_03(); printf("func_1_03\n");
  func_1_04(); printf("func_1_04\n");
  func_1_05(); printf("func_1_05\n");

  // -- CAST --
  func_2_01();            printf("func_2_01\n");
  func_2_02();            printf("func_2_02\n");
  func_2_03();            printf("func_2_03\n");
  func_2_04("func_2_04"); printf("func_2_04\n");

  // -- PARAMETER VALIDATION --
  func_3_01();     printf("func_3_01\n");
  func_3_02();     printf("func_3_02\n");
  func_3_03();     printf("func_3_03\n");
  func_3_04();     printf("func_3_04\n");
  func_3_05();     printf("func_3_05\n");
  func_3_06();     printf("func_3_06\n");
  func_3_07();     printf("func_3_07\n");
  func_3_08();     printf("func_3_08\n");
  func_3_09();     printf("func_3_09\n");
  func_3_10();     printf("func_3_10\n");
  func_3_11();     printf("func_3_11\n");
  func_3_12();     printf("func_3_12\n");
  func_3_13();     printf("func_3_13\n");
  func_3_14();     printf("func_3_14\n");
  func_3_15();     printf("func_3_15\n");
  func_3_16();     printf("func_3_16\n");
  func_3_17();     printf("func_3_17\n");
  func_3_18();     printf("func_3_18\n");
  func_3_19();     printf("func_3_19\n");
  func_3_20();     printf("func_3_20\n");
  func_3_21();     printf("func_3_21\n");
  func_3_22();     printf("func_3_22\n");
  func_3_23();     printf("func_3_23\n");
  func_3_24();     printf("func_3_24\n");
  func_3_25();     printf("func_3_25\n");
  func_3_26();     printf("func_3_26\n");
  func_3_27();     printf("func_3_27\n");
  func_3_28();     printf("func_3_28\n");
  func_3_29();     printf("func_3_29\n");
  func_3_30();     printf("func_3_30\n");
//func_3_31(0);    printf("func_3_31\n");
  func_3_32(2);    printf("func_3_32\n");
  func_3_33(2, 0); printf("func_3_33\n");
  func_3_34(2, 0); printf("func_3_34\n");
//func_3_35(2);    printf("func_3_35\n");
  func_3_36();     printf("func_3_36\n");
  func_3_37();     printf("func_3_37\n");
  func_3_38();     printf("func_3_38\n");

  // -- RETRUN VALUES --
  func_4_01(); printf("func_4_01\n");
  func_4_02(); printf("func_4_02\n");
  func_4_03(); printf("func_4_03\n");

  // -- MEMORY MANAGEMENT --
  func_5_01(); printf("func_5_01\n");
  func_5_02(); printf("func_5_02\n");
  func_5_03(); printf("func_5_03\n");
  func_5_04(); printf("func_5_04\n");
  func_5_05(); printf("func_5_05\n");
  func_5_06(); printf("func_5_06\n");
  func_5_07(); printf("func_5_07\n");
  func_5_08(); printf("func_5_08\n");
  func_5_09(); printf("func_5_09\n");
  func_5_10(); printf("func_5_10\n");
  func_5_11(); printf("func_5_11\n");

  // -- MORE SAFETY FUNCTIONS WITH VALUE RANGE --
  func_6_01(); printf("func_6_01\n");
  func_6_02(); printf("func_6_02\n");
  func_6_03(); printf("func_6_03\n");
  func_6_04(); printf("func_6_04\n");

  // -- FILE ACCESS --
  func_7_01();                         printf("func_7_01\n");
  func_7_02();                         printf("func_7_02\n");
  func_7_03("file1.txt", "file2.txt"); printf("func_7_03\n");
  func_7_04();                         printf("func_7_04\n");
  func_7_05();                         printf("func_7_05\n");
  func_7_06();                         printf("func_7_06\n");
  func_7_07();                         printf("func_7_07\n");
  func_7_08();                         printf("func_7_08\n");
  func_7_09();                         printf("func_7_09\n");
  func_7_10();                         printf("func_7_10\n");
  func_7_11();                         printf("func_7_11\n");
  func_7_12();                         printf("func_7_12\n");
  func_7_13();                         printf("func_7_13\n");
  func_7_14("3914", "At&T+EuA");       printf("func_7_14\n");
  func_7_15();                         printf("func_7_15\n");
  func_7_16();                         printf("func_7_16\n");

  // -- LEAST PRIVILEGES --
  func_8_01();                    printf("func_8_01\n");
  func_8_02(argc, argv[1], argv); printf("func_8_02\n");
  func_8_03();                    printf("func_8_03\n");
  func_8_04();                    printf("func_8_04\n");
  func_8_05(argv);                printf("func_8_05\n");
  func_8_06(argv);                printf("func_8_06\n");
  func_8_07(argv);                printf("func_8_07\n");
  func_8_08(argv);                printf("func_8_08\n");

  // -- CRIPTOGRAFY --
  func_9_01();                    printf("func_9_01\n");

  // -- LIBRABRIES --

  // -- TRY CASH EXCEPTIONS --

  return 0;
}
